'use strict';
/**
 * node-windows-window-controller
 *
 * @author waiting
 * @license MIT
 * @link https://github.com/waitingsong/node-windows-window-controller
 */
Object.defineProperty(exports, '__esModule', { value: true });
const ref = require('ref');
const Win = require('win32-api');
exports.Win = Win;
const win32_api_1 = require('win32-api');
const u32 = require('./lib/user32');
var user32_1 = require('./lib/user32');
exports.validate_cmdshow = user32_1.validate_cmdshow;
const isWin32 = process.platform === 'win32';
const plateformError = 'Invalid platform: win32 required';
const user32 = win32_api_1.U.load();
// expose original user32.ShowWindow() as default
function showWindow(hwnd, nCmdShow) {
    let errcode = 1;
    hwnd = +hwnd;
    if (typeof hwnd !== 'number' || !Number.isSafeInteger(hwnd)) {
        console.error('hwnd must integer');
        return Promise.resolve(errcode);
    }
    if (hwnd <= 0) {
        return Promise.resolve(errcode);
    }
    if (!u32.validate_cmdshow(nCmdShow)) {
        return Promise.resolve(errcode);
    }
    return retrieve_pointer_by_hwnd(hwnd)
        .then(hWnd => {
        if (hWnd && !ref.isNull(hWnd)) {
            if (user32.ShowWindow(hWnd, nCmdShow)) {
                errcode = 0;
            }
        }
        return errcode;
    })
        .catch((err) => {
        return errcode;
    });
}
exports.default = showWindow;
function hide(p, onlyTopWin = true) {
    return proxy(p, 0 /* SW_HIDE */, onlyTopWin);
}
exports.hide = hide;
function show(p, nCmdShow, onlyTopWin = true) {
    return proxy(p, nCmdShow, onlyTopWin);
}
exports.show = show;
// https://msdn.microsoft.com/en-us/library/windows/desktop/ms633548(v=vs.85).aspx
function proxy(p, nCmdShow, onlyTopWin) {
    const execRet = init_execret();
    if (!u32.validate_cmdshow(nCmdShow)) {
        execRet.err = 1;
        execRet.msg = 'value of nCmdShow invalid';
        return Promise.resolve(execRet);
    }
    return get_hwnds(p, onlyTopWin).then(hWnds => {
        if (hWnds && hWnds.length) {
            for (const hWnd of hWnds) {
                if (hWnd && !ref.isNull(hWnd)) {
                    // console.log('hWnd addr:', ref.address(hWnd));
                    u32.show(hWnd, nCmdShow, onlyTopWin)
                        .then((hWnd) => hWnd && !ref.isNull(hWnd) && execRet.hwnds.push(ref.address(hWnd)))
                        .catch(err => {
                        execRet.err = 1;
                        execRet.msg += '\n ' + err;
                    });
                }
            }
        }
    })
        .catch(err => {
        execRet.err = 1;
        execRet.msg += err.toString();
    })
        .then(() => execRet);
}
/**
 * retrieve hWnds by matchValue matched by pid|title
 * all: true means all matched window include child,
 * all: false means only top or owner window
 */
function get_hwnds(p, onlyTopWin) {
    if (!onlyTopWin) {
        return u32.get_hwnds(p);
    }
    else {
        return u32.get_hwnds(p)
            .then((arr) => {
            if (arr && Array.isArray(arr) && arr.length) {
                return u32.filter_main_hwnd(arr);
            }
        });
    }
}
exports.get_hwnds = get_hwnds;
// kill process by which the matched hWnd(s) (window) created
function kill(p) {
    const execRet = init_execret();
    if (typeof p === 'number') {
        _kill(p);
        execRet.pids.push(p);
        return Promise.resolve(execRet);
    }
    else if (typeof p === 'string') {
        return new Promise(resolve => {
            const task = u32.create_task();
            return u32.get_hwnds(p, task).then(() => {
                if (task.pidSet.size) {
                    for (let pid of task.pidSet) {
                        if (_kill(pid)) {
                            execRet.pids.push(pid);
                        }
                    }
                }
                else {
                    execRet.msg = 'the pid list to be killed empty';
                }
                resolve(execRet);
            });
        });
    }
    else {
        execRet.err = 1;
        execRet.msg = 'kill() param must be either number or string';
        return Promise.resolve(execRet);
    }
}
exports.kill = kill;
function _kill(pid) {
    try {
        process.kill(pid, 0) && process.kill(pid);
        return true;
    }
    catch (ex) {
        console.error(ex);
    }
    return false;
}
function init_execret() {
    return {
        err: 0,
        msg: '',
        pids: [],
        hwnds: [],
    };
}
// retrive hWnd buffer from decimal/hex value of hWnd
function retrieve_pointer_by_hwnd(hwnd) {
    const task = u32.create_task();
    task.matchType = 'hwnd';
    return u32.get_hwnds(hwnd, task)
        .then(arr => {
        if (arr && arr.length) {
            return arr[0];
        }
    });
}
exports.retrieve_pointer_by_hwnd = retrieve_pointer_by_hwnd;
